@@ -9,7 +9,7 @@
 #ifndef __XML_ENTITIES_H__
 #define __XML_ENTITIES_H__
 
-#include "parser.h"
+#include "tree.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -34,6 +34,7 @@ typedef struct xmlEntity {
     const CHAR    *ExternalID;	/* External identifier for PUBLIC Entity */
     const CHAR    *SystemID;	/* URI for a SYSTEM or PUBLIC Entity */
     CHAR *content;		/* The entity content or ndata if unparsed */
+    CHAR *orig;			/* The entity cont without ref substitution */
 } xmlEntity;
 typedef xmlEntity *xmlEntityPtr;
 
@@ -56,6 +57,8 @@ typedef xmlEntitiesTable *xmlEntitiesTablePtr;
  * External functions :
  */
 
+#include "parser.h"
+
 void xmlAddDocEntity(xmlDocPtr doc, const CHAR *name, int type,
               const CHAR *ExternalID, const CHAR *SystemID, CHAR *content);
 void xmlAddDtdEntity(xmlDocPtr doc, const CHAR *name, int type,
@@ -67,9 +70,7 @@ CHAR *xmlEncodeEntities(xmlDocPtr doc, const CHAR *input);
 xmlEntitiesTablePtr xmlCreateEntitiesTable(void);
 xmlEntitiesTablePtr xmlCopyEntitiesTable(xmlEntitiesTablePtr table);
 void xmlFreeEntitiesTable(xmlEntitiesTablePtr table);
-void xmlDumpEntitiesTable(xmlEntitiesTablePtr table);
-xmlParserInputPtr xmlNewEntityInputStream(xmlParserCtxtPtr ctxt,
-                                                 xmlEntityPtr entity);
+void xmlDumpEntitiesTable(xmlBufferPtr buf, xmlEntitiesTablePtr table);
 xmlEntitiesTablePtr xmlCopyEntitiesTable(xmlEntitiesTablePtr table);
 
 #ifdef __cplusplus
